// set.hpp
//
// Copyright (C) 2006 Visual Media FX Ltd.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef SET_INC_
#define SET_INC_

#include <vector>
#include <boost/shared_ptr.hpp>
#include <openassetlib/al/asset.hpp>

namespace olib { namespace openassetlib { namespace al {

typedef boost::shared_ptr<class set>	set_ptr;

class AL_DECLSPEC set
{
public:
	typedef std::vector<asset_ptr>					asset_vector;
	typedef asset_vector::value_type				value_type;
	typedef asset_vector::iterator					iterator;
	typedef asset_vector::const_iterator			const_iterator;
	typedef asset_vector::reverse_iterator			reverse_iterator;
	typedef asset_vector::const_reverse_iterator	const_reverse_iterator;
	typedef asset_vector::reference					reference;
	typedef asset_vector::const_reference			const_reference;
	typedef asset_vector::size_type					size_type;

	static set_ptr create(const opl::string& name = "");
	static set_ptr create(const asset_ptr& asset, const opl::string& name = "");
	static set_ptr create(const set_ptr& set, const opl::string& name = "");

	virtual ~set(void);

	opl::string		name() const					{	return name_;	};
	void			name(const opl::string& name)	{	name_ = name;	};

	const_reference					at(size_type pos) const											{	return vec_.at(pos);				};
	reference						at(size_type pos)												{	return vec_.at(pos);				};
	const_reference					back() const													{	return vec_.back();					};
	reference						back()															{	return vec_.back();					};
	const_iterator					begin() const													{	return vec_.begin();				};
	iterator						begin()															{	return vec_.begin();				};
	size_type						capacity() const												{	return vec_.capacity();				};
	void							clear()															{	vec_.clear();						};
	bool							empty() const													{	return vec_.empty();				};
	const_iterator					end() const														{	return vec_.end();					};
	iterator						end()															{	return vec_.end();					};
	iterator						erase(iterator where)											{	return vec_.erase(where);			};
	iterator						erase(iterator first, iterator last)							{	return vec_.erase(first, last);		};
	const_reference					front() const													{	return vec_.front();				};
	reference						front()															{	return vec_.front();				};
	template<class iterator> void	insert(iterator where, iterator first, iterator last)			{	vec_.insert(where, first, last);	};
	iterator						insert(iterator where, const value_type& val)					{	return vec_.insert(where, val);		};
	void							insert(iterator where, size_type count, const value_type& val)	{	vec_.insert(where, count, val);		};
	size_type						max_size() const												{	return vec_.max_size();				};
	void							pop_back()														{	vec_.pop_back();					};
	void							push_back(const value_type& val)								{	vec_.push_back(val);				};
	const_reverse_iterator			rbegin() const													{	return vec_.rbegin();				};
	reverse_iterator				rbegin()														{	return vec_.rbegin();				};
	reverse_iterator				rend()															{	return vec_.rend();					};
	const_reverse_iterator			rend() const													{	return vec_.rend();					};
	void							resize(size_type newsize)										{	vec_.resize(newsize);				};
	void							resize(size_type newsize, value_type val)						{	vec_.resize(newsize, val);			};
	void							reserve(size_type count)										{	vec_.reserve(count);				};
	size_type						size() const													{	return vec_.size();					};
	void							swap(set& right)												{	vec_.swap(right.vec_);				};
	const_reference					operator[](size_type pos) const									{	return vec_.operator[](pos);		};
	reference						operator[](size_type pos)										{	return vec_.operator[](pos);		};

	// additional helper functions to be exported to python interface
	void remove(const asset_ptr& asset);
	bool insert(const asset_ptr& asset, int offset);
	bool insert(const set_ptr& set, int offset);
	asset_ptr find(const opl::string& guid);

private:
	set(const opl::string& name = "");
	set(const asset_ptr asset, const opl::string& name = "");
	set(const set_ptr set, bool deepcopy, const opl::string& name = "");

	opl::string		name_;
	asset_vector	vec_;
};

} } }

#endif //SET_INC_
