
// fx - A compositing library representation.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef N_OPERATOR_GPU_KERNEL_INC_
#define N_OPERATOR_GPU_KERNEL_INC_

#ifdef HAVE_CONFIG_H
#include <openlibraries_global_config.hpp>
#endif

#include <boost/filesystem/path.hpp>

#include <openpluginlib/pl/shader_manager.hpp>
#include <openpluginlib/pl/cg.hpp>

#include <openeffectslib/fx/config.hpp>
#include <openeffectslib/fx/n_operator_gpu.hpp>

namespace olib { namespace openeffectslib { namespace fx {

//
class FX_DECLSPEC n_operator_gpu_kernel : public fx::n_operator_gpu
{
private:
	struct FX_DECLSPEC shader_object
	{
		explicit shader_object( )
			: GLSL_valid( false )
			, GLSL_tried( false )
			, Cg_valid( false )
			, Cg_tried( false )
		{ }
		
		openpluginlib::GLSL_shader_object GLSL_shader;
		GLint GLSL_InColor0;
		
#if defined( HAVE_CG_RUNTIME )
		openpluginlib::Cg_shader_object Cg_shader;
		
		CGparameter Cg_InColor0;
		CGparameter model_view_proj;
#endif
		bool GLSL_valid;
		bool GLSL_tried;
		bool Cg_valid;
		bool Cg_tried;
	};
	
public:
	explicit n_operator_gpu_kernel( const boost::filesystem::path& path );
	virtual ~n_operator_gpu_kernel( );

public:
	virtual bool init( );
	virtual bool update_image( );
	bool is_GLSL_valid( ) const { return sobj_.GLSL_valid; }
	bool is_Cg_valid( )   const	{ return sobj_.Cg_valid; }
	
private:
	virtual bool start( );
	virtual bool finish( );

private:
#if defined ( HAVE_CG_RUNTIME )
	bool download_Cg_shader( );
	void enable_Cg_shader( );
	void disable_Cg_shader( );
#endif
	bool download_GLSL_shader( );
	void enable_GLSL_shader( );
	void disable_GLSL_shader( );
	
private:
	shader_object sobj_;
	
private:
	bool download_shader( );
	bool valid( ) const;
	
private:
	//openpluginlib::shader_object_ptr obj_;
};

} } }

#endif
