
// oel - An effects library representation.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef OEL_PLAYABLE_H_
#define OEL_PLAYABLE_H_

#include <openeffectslib/fx/n_operator.hpp>

namespace olib { namespace openeffectslib { namespace fx {

// The playable interface provides an optional interface that allows any operator
// to be seekable, but also allows it to expose different operators for both audio
// and image, or if need be, for a given frame position
//
// The intention is that a source operator can encapsulate normalising filters 
// (such as colour space converters, deinterlacers, audio resamplers and so on), but
// still be seen to the connector as a single seekable entity. 
//
// Further, it can enapsulate multiple source operators transparently, thus enabling
// playlist and track type of functionality directly in a single operator.

class playable
{
public:
	virtual ~playable( ) { }
	// Number of frames
	virtual int length( ) const = 0;
	// Number of frames per second
	virtual double fps( ) const = 0;
	// Current position (this is a convenience function - it's normally equivalent to
	// what the user is currently seeing, hence it normally just calls position_image)
	virtual int position( ) const = 0;
	// The image position
	virtual int position_image( ) const = 0;
	// The audio position - typically during playout, this will be ahead of the image 
	// position
	virtual int position_audio( ) const = 0;
	// Seek on both image and audio streams
	virtual void seek( int position, bool relative = false ) = 0;
	// Seek on image
	virtual void seek_image( int position, bool relative = false ) = 0;
	// Seek on audio
	virtual void seek_audio( int position, bool relative = false ) = 0;
};

// Convert an operator to playable if it implements it
inline playable *make_playable( n_operator_ptr op )
{ return dynamic_cast< playable * >( op.get( ) ); }

} } }

#endif
