
// ml - A media library representation.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef OPENMEDIALIB_AUDIO_INC_
#define OPENMEDIALIB_AUDIO_INC_

#include <openmedialib/ml/config.hpp>
#include <openimagelib/il/basic_image.hpp>
#include <boost/shared_ptr.hpp>

namespace olib { namespace openmedialib { namespace ml {

template<typename T, class storage = olib::openimagelib::il::default_storage<T> >
class audio_format : public storage
{
public:
	typedef typename storage::const_pointer const_pointer;
	typedef typename storage::pointer		pointer;
	typedef typename storage::size_type		size_type;

protected:
	explicit audio_format( size_type frequency,
						   size_type channels,
						   size_type samples,
						   std::wstring af )
		: frequency_( frequency )
		, channels_( channels )
		, samples_( samples )
		, af_( af )
	{ }

public:			
	virtual ~audio_format( )
	{ }

	virtual audio_format *clone( ) = 0;

protected:
	void allocate( )
	{
		size_type siz = allocsize( );
		storage::create( siz );
	}

public:
	size_type 		frequency( )		const { return frequency_; }
	size_type 		channels( )			const { return channels_; }
	size_type 		samples( )			const { return samples_; }
	std::wstring	af( )				const { return af_; }

public:
	virtual size_type allocsize( ) const = 0;

private:
	size_type		frequency_;
	size_type		channels_;
	size_type		samples_;
	std::wstring 	af_;
};

template<typename T, template<class, class> class structure, template<class> class storage = olib::openimagelib::il::default_storage>
class audio
{
public:
	typedef typename storage<T>::const_pointer				const_pointer;
	typedef typename storage<T>::pointer					pointer;
	typedef typename structure<T, storage<T> >::size_type	size_type;

	explicit audio( size_type frequency,
					size_type channels,
					size_type samples )
		: structure_( new structure<T, storage<T> >( frequency, channels, samples ) )
		, pts_( 0 )
		, position_( 0 )
	{ }

	// Allocate an audio based on the audio size but don't populate
	template<typename U, template<class, class> class V, template<class> class W>
	audio( const audio<U, V, W>& other )
		: structure_( new V<U, W<U> >( other.frequency( ), other.channels( ), other.samples( ) ) )
		, pts_( other.pts( ) )
		, position_( other.position( ) )
	{ memcpy( data( ), other.data( ), structure_->size( ) ); }

public:
	audio *clone( )
	{
		return new audio( *this );
	}

	size_type 		frequency( ) const { return structure_->frequency( ); }
	size_type 		channels( )	 const { return structure_->channels( ); }
	size_type 		samples( )	 const { return structure_->samples( ); }
	std::wstring	af( )		 const { return structure_->af( ); }

	const_pointer 	data( ) 	 const { return structure_->data( ); }
	pointer 		data( ) 	 	   { return structure_->data( ); }

	double pts( )				 const { return pts_; }
	void set_pts( double pts )		   { pts_ = pts; }

	int position( )				 const { return position_; }
	void set_position( int position )  { position_ = position; }

	size_type size( ) const		 { return structure_->size( ); }

public:
	size_type allocsize( ) const { return structure_->allocsize( ); }

private:
	boost::shared_ptr<structure<T, storage<T> > > structure_;
	double pts_;
	int position_;
};

typedef ML_DECLSPEC audio<unsigned char, audio_format>	  audio_type;
typedef ML_DECLSPEC boost::shared_ptr<audio_type> audio_type_ptr;

template<typename T, class storage = olib::openimagelib::il::default_storage<T> >
class pcm16 : public audio_format<T, storage>
{
public:
	typedef typename storage::const_pointer					const_pointer;
	typedef typename storage::pointer						pointer;
	typedef typename audio_format<T, storage>::size_type	size_type;

private:
	static const int sample_size = 2;

public:
	explicit pcm16(	size_type frequency,
					size_type channels,
					size_type samples )
		: audio_format<T, storage>( frequency, channels, samples, L"pcm16" )
	{ audio_format<T, storage>::allocate( ); }

	pcm16( const pcm16& other )
		: audio_format<T, storage>( other.frequency( ), other.channels( ), other.samples( ), L"pcm16" )
	{ audio_format<T, storage>::allocate( ); }

	virtual ~pcm16( )
	{ }

public:
	virtual size_type allocsize( ) const
	{ return sample_size * audio_format<T, storage>::samples( ) * audio_format<T, storage>::channels( ); }
	
	virtual pcm16* clone( )
	{ return new pcm16( *this ); }
};

} } }

#endif
