
// ml - A media library representation.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef OPENMEDIALIB_FRAME_INC_
#define OPENMEDIALIB_FRAME_INC_

#include <openimagelib/il/il.hpp>
#include <openmedialib/ml/audio.hpp>
#include <boost/shared_ptr.hpp>

namespace olib { namespace openmedialib { namespace ml {

typedef ML_DECLSPEC olib::openimagelib::il::image_type image_type;
typedef ML_DECLSPEC olib::openimagelib::il::image_type_ptr image_type_ptr;

class ML_DECLSPEC frame_type 
{
	public:
		explicit frame_type( ) 
			: image_( )
			, audio_( )
			, pts_( 0.0 )
			, position_( 0 )
			, duration_( 0.0 )
			, num_( 1 )
			, den_( 1 )
		{ }

		virtual ~frame_type( ) 
		{ }

		virtual void set_image( image_type_ptr image ) { image_ = image; }
		virtual image_type_ptr get_image( ) { return image_; }
		virtual void set_audio( audio_type_ptr audio ) { audio_ = audio; }
		virtual audio_type_ptr get_audio( ) { return audio_; }
		virtual void set_pts( double pts ) { pts_ = pts; }
		virtual double get_pts( ) const { return pts_; }
		virtual void set_position( int position ) { position_ = position; }
		virtual int get_position( ) const { return position_; }
		virtual void set_duration( double duration ) { duration_ = duration; }
		virtual double get_duration( ) const { return duration_; }
		virtual void set_sar( int num, int den ) { num_ = num; den_ = den; }
		virtual void get_sar( int &num, int &den ) const { num = num_; den = den_; }

		virtual double aspect_ratio( )
		{
			int num, den;
			get_sar( num, den );
			if ( num == 0 )
				return ( double )get_image( )->width( ) / get_image( )->height( );
			else
				return ( ( double ) num / den ) * ( ( double )get_image( )->width( ) / get_image( )->height( ) );
		}

	private:
		image_type_ptr image_;
		audio_type_ptr audio_;
		double pts_;
		int position_;
		double duration_;
		int num_;
		int den_;
};

typedef ML_DECLSPEC boost::shared_ptr< frame_type > frame_type_ptr;

} } }

#endif
