
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.

#ifndef PCOS_PROPERTY_H
#define PCOS_PROPERTY_H

#include <openpluginlib/pl/config.hpp>

#include <openpluginlib/pl/pcos/subject.hpp>
#include <openpluginlib/pl/pcos/key.hpp>
#include <openpluginlib/pl/pcos/any.hpp>

// boost
#include <boost/shared_ptr.hpp>

namespace olib { namespace openpluginlib { namespace pcos {

class any;

/// A property class i.e. a pairing of a key to a value.
/// Properties are reference counted, and so may be passed by 
/// value if desired without much overhead.
class OPENPLUGINLIB_DECLSPEC property : public subject
{
public:
    static property NULL_PROPERTY;

    property( const key& );
    virtual ~property();

    // allow retrieval of the key
    key getKey() const;

    /// allow direct assignment
    property& operator=( const any& );
    property& operator=( const property& );

    /// equality check; impl pointer comparison is enough
    bool operator==( const property& );
    bool operator!=( const property& );

    /// return the value; will throw a bad_any_cast if 
    /// we try to extract the wrong value type
    template < typename T > T value() const;
    
    /// query the type held by the property
    template < typename T > bool isA() const;

    /// always notify any observers on an assignment
    /// (by default, notification only happens if the assigned
    /// value and the current value are different)
    void setAlwaysNotify( bool );

private:
    property();

    class property_impl;
    boost::shared_ptr< property_impl > m_impl;
};


} } }

#endif // PCOS_PROPERTY_H
