
// openpluginlib - A plugin interface to openlibraries.

// Copyright (C) 2005-2006 VM Inc.
// Released under the LGPL.
// For more information, see http://www.openlibraries.org.


#ifndef PCOS_PROPERTY_CONTAINER_H
#define PCOS_PROPERTY_CONTAINER_H

#include <openpluginlib/pl/config.hpp>

// framework
#include <openpluginlib/pl/pcos/property.hpp>
#include <openpluginlib/pl/pcos/key.hpp>
#include <openpluginlib/pl/pcos/subject.hpp>

// boost
#include <boost/shared_ptr.hpp>

namespace olib { namespace openpluginlib { namespace pcos {

/// A loose container of properties; aggregates property notification
/// i.e. if a contained property changes, so the property_container
/// will notify it has changed
class OPENPLUGINLIB_DECLSPEC property_container : public subject
{
public:
    property_container();

    property getProperty( const char* ) const;
    property getProperty( const key& ) const;

    /// return a list of all known property keys
    key_vector getKeys() const;

    /// add a property to the collection
    void append( const property& );

    /// remove a property from the collection
    void remove( const property& );

private:
    class property_container_impl;
    boost::shared_ptr< property_container_impl > m_impl;
};

} } }

#endif // PCOS_PROPERTY_CONTAINER_H
